########################################################
# Genetic Mapping Script
# Written by: Michael A. Hardigan
# Annotated by: Dominique D.A. Pincot
# Time Ratings: Fast (<5 minutes); Medium (5-45 minutes); Slow (>45 minutes) [rough estimates for < 10k markers]
########################################################

detach(package:BatchMap)
library(onemap)

setwd(dir="/path/to/mapping/directory/")

########################################################
# R: Binning of Markers & Subgroup (Chromosomal Fragments) Identification 
# Time: Fast
########################################################

#First, save a copy of your input file ("genotypes.<parent_id>.onemap.txt") as "ld_comp.marker_gntps.txt"

# read in preprocessed genotype files; these will need to be formatted to meet onemap's specifications
crossobj <- read_onemap(inputfile = "genotypes.<parent_id>.onemap.txt")

bins <- find_bins(crossobj,exact=FALSE)

# create binned outcross object
crossobj_binned <- create_data_bins(crossobj,bins)

# print co-segregating marker bins
sink("cosegregating_markers.<parent_id>.outcross.txt")
print(bins$bins)
sink()

# calculate pairwise recombination fractions
crossobj_2pt <- rf_2pts(crossobj_binned,LOD=8,max.rf=0.18)

# create marker linkage groups - BatchMap
crossobj_groups <- group(make_seq(input.obj=crossobj_2pt,"all"),LOD=10,max.rf=0.08)

# print group markers for LD-comp 
crossobj_groups

#copy group summary to output file "<parent_id>.subgroups.onemap.txt"

########################################################
# Perl: Preprocessing
# Time: Fast
########################################################
#Navigate to the folder containing your above files
#Run the following line in the PERL program of your choice:
# perl .\script_01.prep_ld_comp.pl --groups <parent_id>.subgroups.onemap.txt 
#this creates two files: "runcommands.txt" and "ld_comp.marker_lgs.txt"
#"ld_comp.marker_lgs.txt" is an transient key file that links marker to their initial subgroup
#convert "runcommands.txt" to a bash .sh file by opening it in a text editor and adding the following at the top of the text:
# "#!/bin/bash <enter>#"  , where <enter> is an actual line break and the quotations are not included
#You will need to change the .txt extension to a .sh extension too
#This file will calculate the LD statistics of each subgroup with every other subgroup
#for the next step, you will need the two files above, and "ld.comp.marker_gntps.txt" (a copy of the original genotype file)


########################################################
# Bash: Subgroup LD statistics and grouping
# Time: Slow
########################################################
#run "runcommands.sh"; this iterates through the subgroups and generates key information using perl script "script_02.test_lg_pairwise_linkage.pl"
#run the following command to save output:
#save output cat tempout/* | sort -k7rn,7 -k8rn,8 -k9rn,9 -k10rn,10 -k11rn,11 -k12rn,12 | head -n 1000 > output.txt
#This saves the top 1000 'closely linked' results (see next section for more information) into a text file
#this text files shows the subgroups, the number of markers in those subgroups, and the LD statistics with other subgroups


########################################################
# Excel (or similar program): Manual Grouping Step
# Time: Medium or Slow
########################################################
#open "output.txt" from step above
#Column order is: Group1Name, Group1%Identity, Group1Marker#, Group2Name, Group2%Identity, Group2Marker#, Top LD Value, Mean(Top 5 LD Values), Mean(Top 10 LD Values), Mean(Top 50 LD Values), Mean(Top 100 LD Values), Mean(Top 200 LD Values),Mean(Top 500 LD Values), Mean(Top 2500 LD Values), Identicals
#Note: The "identity" columns are only valid if you've named your genetic markers as Chromosome_position in the original genotype file
#Based on LD statistics and identity, group subgroups together into new linkage groups
#make a text file "ld_comp.lg_chroms.txt" that has one column for subgroup IDs (eg: group_01) and another column for the linkage group you are assigning it to (eg: LG1_1)


########################################################
# Perl: Input File Generation
# Time: Fast
########################################################
# run perl script "script_05.print_chrspec_onemap_input.pl"
# this generates chromosome/linkage-group specific input files from the previous files you have generated: "ld_comp.marker_gntps.txt", "ld_comp.marker_lg.txt", "ld_comp.lg_chroms.txt"


########################################################
# R: Chromosome-specific Marker Data Input
# Time: Fast
########################################################

setwd(dir="/path/to/mapping/directory/")

# read in input marker genotype files for chromosome/linkage-group-specific linkage groups
outcross_01 <- read_onemap(inputfile = "genotypes.LG1_1.onemap.txt")
outcross_02 <- read_onemap(inputfile = "genotypes.LG1_2.onemap.txt")
outcross_03 <- read_onemap(inputfile = "genotypes.LG1_3.onemap.txt")
outcross_04 <- read_onemap(inputfile = "genotypes.LG1_4.onemap.txt")
outcross_05 <- read_onemap(inputfile = "genotypes.LG2_1.onemap.txt")
outcross_06 <- read_onemap(inputfile = "genotypes.LG2_2.onemap.txt")
outcross_07 <- read_onemap(inputfile = "genotypes.LG2_3.onemap.txt")
outcross_08 <- read_onemap(inputfile = "genotypes.LG2_4.onemap.txt")
outcross_09 <- read_onemap(inputfile = "genotypes.LG3_1.onemap.txt")
outcross_10 <- read_onemap(inputfile = "genotypes.LG3_2.onemap.txt")
outcross_11 <- read_onemap(inputfile = "genotypes.LG3_3.onemap.txt")
outcross_12 <- read_onemap(inputfile = "genotypes.LG3_4.onemap.txt")
outcross_13 <- read_onemap(inputfile = "genotypes.LG4_1.onemap.txt")
outcross_14 <- read_onemap(inputfile = "genotypes.LG4_2.onemap.txt")
outcross_15 <- read_onemap(inputfile = "genotypes.LG4_3.onemap.txt")
outcross_16 <- read_onemap(inputfile = "genotypes.LG4_4.onemap.txt")
outcross_17 <- read_onemap(inputfile = "genotypes.LG5_1.onemap.txt")
outcross_18 <- read_onemap(inputfile = "genotypes.LG5_2.onemap.txt")
outcross_19 <- read_onemap(inputfile = "genotypes.LG5_3.onemap.txt")
outcross_20 <- read_onemap(inputfile = "genotypes.LG5_4.onemap.txt")
outcross_21 <- read_onemap(inputfile = "genotypes.LG6_1.onemap.txt")
outcross_22 <- read_onemap(inputfile = "genotypes.LG6_2.onemap.txt")
outcross_23 <- read_onemap(inputfile = "genotypes.LG6_3.onemap.txt")
outcross_24 <- read_onemap(inputfile = "genotypes.LG6_4.onemap.txt")
outcross_25 <- read_onemap(inputfile = "genotypes.LG7_1.onemap.txt")
outcross_26 <- read_onemap(inputfile = "genotypes.LG7_2.onemap.txt")
outcross_27 <- read_onemap(inputfile = "genotypes.LG7_3.onemap.txt")
outcross_28 <- read_onemap(inputfile = "genotypes.LG7_4.onemap.txt")

detach(package:onemap)
library(BatchMap)

# calculate pairwise recombination fractions - BatchMap
outcross_2pt_01 <- rf.2pts(outcross_01,LOD=5,max.rf=0.50,verbose=TRUE)
outcross_2pt_02 <- rf.2pts(outcross_02,LOD=5,max.rf=0.50,verbose=TRUE)
outcross_2pt_03 <- rf.2pts(outcross_03,LOD=5,max.rf=0.50,verbose=TRUE)
outcross_2pt_04 <- rf.2pts(outcross_04,LOD=5,max.rf=0.50,verbose=TRUE)
outcross_2pt_05 <- rf.2pts(outcross_05,LOD=5,max.rf=0.50,verbose=TRUE)
outcross_2pt_06 <- rf.2pts(outcross_06,LOD=5,max.rf=0.50,verbose=TRUE)
outcross_2pt_07 <- rf.2pts(outcross_07,LOD=5,max.rf=0.50,verbose=TRUE)
outcross_2pt_08 <- rf.2pts(outcross_08,LOD=5,max.rf=0.50,verbose=TRUE)
outcross_2pt_09 <- rf.2pts(outcross_09,LOD=5,max.rf=0.50,verbose=TRUE)
outcross_2pt_10 <- rf.2pts(outcross_10,LOD=5,max.rf=0.50,verbose=TRUE)
outcross_2pt_11 <- rf.2pts(outcross_11,LOD=5,max.rf=0.50,verbose=TRUE)
outcross_2pt_12 <- rf.2pts(outcross_12,LOD=5,max.rf=0.50,verbose=TRUE)
outcross_2pt_13 <- rf.2pts(outcross_13,LOD=5,max.rf=0.50,verbose=TRUE)
outcross_2pt_14 <- rf.2pts(outcross_14,LOD=5,max.rf=0.50,verbose=TRUE)
outcross_2pt_15 <- rf.2pts(outcross_15,LOD=5,max.rf=0.50,verbose=TRUE)
outcross_2pt_16 <- rf.2pts(outcross_16,LOD=5,max.rf=0.50,verbose=TRUE)
outcross_2pt_17 <- rf.2pts(outcross_17,LOD=5,max.rf=0.50,verbose=TRUE)
outcross_2pt_18 <- rf.2pts(outcross_18,LOD=5,max.rf=0.50,verbose=TRUE)
outcross_2pt_19 <- rf.2pts(outcross_19,LOD=5,max.rf=0.50,verbose=TRUE)
outcross_2pt_20 <- rf.2pts(outcross_20,LOD=5,max.rf=0.50,verbose=TRUE)
outcross_2pt_21 <- rf.2pts(outcross_21,LOD=5,max.rf=0.50,verbose=TRUE)
outcross_2pt_22 <- rf.2pts(outcross_22,LOD=5,max.rf=0.50,verbose=TRUE)
outcross_2pt_23 <- rf.2pts(outcross_23,LOD=5,max.rf=0.50,verbose=TRUE)
outcross_2pt_24 <- rf.2pts(outcross_24,LOD=5,max.rf=0.50,verbose=TRUE)
outcross_2pt_25 <- rf.2pts(outcross_25,LOD=5,max.rf=0.50,verbose=TRUE)
outcross_2pt_26 <- rf.2pts(outcross_26,LOD=5,max.rf=0.50,verbose=TRUE)
outcross_2pt_27 <- rf.2pts(outcross_27,LOD=5,max.rf=0.50,verbose=TRUE)
outcross_2pt_28 <- rf.2pts(outcross_28,LOD=5,max.rf=0.50,verbose=TRUE)

# create marker linkage groups - BatchMap
grouped_01 <- group(make.seq(input.obj=outcross_2pt_01,"all"),LOD=5,max.rf=0.50)
grouped_02 <- group(make.seq(input.obj=outcross_2pt_02,"all"),LOD=5,max.rf=0.50)
grouped_03 <- group(make.seq(input.obj=outcross_2pt_03,"all"),LOD=5,max.rf=0.50)
grouped_04 <- group(make.seq(input.obj=outcross_2pt_04,"all"),LOD=5,max.rf=0.50)
grouped_05 <- group(make.seq(input.obj=outcross_2pt_05,"all"),LOD=5,max.rf=0.50)
grouped_06 <- group(make.seq(input.obj=outcross_2pt_06,"all"),LOD=5,max.rf=0.50)
grouped_07 <- group(make.seq(input.obj=outcross_2pt_07,"all"),LOD=5,max.rf=0.50)
grouped_08 <- group(make.seq(input.obj=outcross_2pt_08,"all"),LOD=5,max.rf=0.50)
grouped_09 <- group(make.seq(input.obj=outcross_2pt_09,"all"),LOD=5,max.rf=0.50)
grouped_10 <- group(make.seq(input.obj=outcross_2pt_10,"all"),LOD=5,max.rf=0.50)
grouped_11 <- group(make.seq(input.obj=outcross_2pt_11,"all"),LOD=5,max.rf=0.50)
grouped_12 <- group(make.seq(input.obj=outcross_2pt_12,"all"),LOD=5,max.rf=0.50)
grouped_13 <- group(make.seq(input.obj=outcross_2pt_13,"all"),LOD=5,max.rf=0.50)
grouped_14 <- group(make.seq(input.obj=outcross_2pt_14,"all"),LOD=5,max.rf=0.50)
grouped_15 <- group(make.seq(input.obj=outcross_2pt_15,"all"),LOD=5,max.rf=0.50)
grouped_16 <- group(make.seq(input.obj=outcross_2pt_16,"all"),LOD=5,max.rf=0.50)
grouped_17 <- group(make.seq(input.obj=outcross_2pt_17,"all"),LOD=5,max.rf=0.50)
grouped_18 <- group(make.seq(input.obj=outcross_2pt_18,"all"),LOD=5,max.rf=0.50)
grouped_19 <- group(make.seq(input.obj=outcross_2pt_19,"all"),LOD=5,max.rf=0.50)
grouped_20 <- group(make.seq(input.obj=outcross_2pt_20,"all"),LOD=5,max.rf=0.50)
grouped_21 <- group(make.seq(input.obj=outcross_2pt_21,"all"),LOD=5,max.rf=0.50)
grouped_22 <- group(make.seq(input.obj=outcross_2pt_22,"all"),LOD=5,max.rf=0.50)
grouped_23 <- group(make.seq(input.obj=outcross_2pt_23,"all"),LOD=5,max.rf=0.50)
grouped_24 <- group(make.seq(input.obj=outcross_2pt_24,"all"),LOD=5,max.rf=0.50)
grouped_25 <- group(make.seq(input.obj=outcross_2pt_25,"all"),LOD=5,max.rf=0.50)
grouped_26 <- group(make.seq(input.obj=outcross_2pt_26,"all"),LOD=5,max.rf=0.50)
grouped_27 <- group(make.seq(input.obj=outcross_2pt_27,"all"),LOD=5,max.rf=0.50)
grouped_28 <- group(make.seq(input.obj=outcross_2pt_28,"all"),LOD=5,max.rf=0.50)

#If any of the grouped objects above show two or three groups, return to the "ld_comp.lg_chroms.txt" file and divide the groups into A and B subsections of the chromosome (eg: LG2_4 to LG2_4a and LG2_4b)
#Note: It may not make sense to keep some of these split out groups (eg: if there are only 2 markers on that subgroup)
#make sure to rerun perl script "script_05.print_chrspec_onemap_input.pl" if you do split groups and to rerun the Batchmap code above

########################################################
# R: Chromosome-specific Marker Mapping
# Time: Slow
########################################################

# generate marker sequences - BatchMap
grouped_seq_01 <- make.seq(grouped_01,1)
grouped_seq_02 <- make.seq(grouped_02,1)
grouped_seq_03 <- make.seq(grouped_03,1)
grouped_seq_04 <- make.seq(grouped_04,1)
grouped_seq_05 <- make.seq(grouped_05,1)
grouped_seq_06 <- make.seq(grouped_06,1)
grouped_seq_07 <- make.seq(grouped_07,1)
grouped_seq_08 <- make.seq(grouped_08,1)
grouped_seq_09 <- make.seq(grouped_09,1)
grouped_seq_10 <- make.seq(grouped_10,1)
grouped_seq_11 <- make.seq(grouped_11,1)
grouped_seq_12 <- make.seq(grouped_12,1)
grouped_seq_13 <- make.seq(grouped_13,1)
grouped_seq_14 <- make.seq(grouped_14,1)
grouped_seq_15 <- make.seq(grouped_15,1)
grouped_seq_16 <- make.seq(grouped_16,1)
grouped_seq_17 <- make.seq(grouped_17,1)
grouped_seq_18 <- make.seq(grouped_18,1)
grouped_seq_19 <- make.seq(grouped_19,1)
grouped_seq_20 <- make.seq(grouped_20,1)
grouped_seq_21 <- make.seq(grouped_21,1)
grouped_seq_22 <- make.seq(grouped_22,1)
grouped_seq_23 <- make.seq(grouped_23,1)
grouped_seq_24 <- make.seq(grouped_24,1)
grouped_seq_25 <- make.seq(grouped_25,1)
grouped_seq_26 <- make.seq(grouped_26,1)
grouped_seq_27 <- make.seq(grouped_27,1)
grouped_seq_28 <- make.seq(grouped_28,1)


# estimate initial marker orderings - BatchMap; you can allocate more cores to this to hasten process if you have them
grouped_01_order <- record.parallel(grouped_seq_01,times=48,cores=6)
grouped_02_order <- record.parallel(grouped_seq_02,times=48,cores=6)
grouped_03_order <- record.parallel(grouped_seq_03,times=48,cores=6)
grouped_04_order <- record.parallel(grouped_seq_04,times=48,cores=6)
grouped_05_order <- record.parallel(grouped_seq_05,times=48,cores=6)
grouped_06_order <- record.parallel(grouped_seq_06,times=48,cores=6)
grouped_07_order <- record.parallel(grouped_seq_07,times=48,cores=6)
grouped_08_order <- record.parallel(grouped_seq_08,times=48,cores=6)
grouped_09_order <- record.parallel(grouped_seq_09,times=48,cores=6)
grouped_10_order <- record.parallel(grouped_seq_10,times=48,cores=6)
grouped_11_order <- record.parallel(grouped_seq_11,times=48,cores=6)
grouped_12_order <- record.parallel(grouped_seq_12,times=48,cores=6)
grouped_13_order <- record.parallel(grouped_seq_13,times=48,cores=6)
grouped_14_order <- record.parallel(grouped_seq_14,times=48,cores=6)
grouped_15_order <- record.parallel(grouped_seq_15,times=48,cores=6)
grouped_16_order <- record.parallel(grouped_seq_16,times=48,cores=6)
grouped_17_order <- record.parallel(grouped_seq_17,times=48,cores=6)
grouped_18_order <- record.parallel(grouped_seq_18,times=48,cores=6)
grouped_19_order <- record.parallel(grouped_seq_19,times=48,cores=6)
grouped_20_order <- record.parallel(grouped_seq_20,times=48,cores=6)
grouped_21_order <- record.parallel(grouped_seq_21,times=48,cores=6)
grouped_22_order <- record.parallel(grouped_seq_22,times=48,cores=6)
grouped_23_order <- record.parallel(grouped_seq_23,times=48,cores=6)
grouped_24_order <- record.parallel(grouped_seq_24,times=48,cores=6)
grouped_25_order <- record.parallel(grouped_seq_25,times=48,cores=6)
grouped_26_order <- record.parallel(grouped_seq_26,times=48,cores=6)
grouped_27_order <- record.parallel(grouped_seq_27,times=48,cores=6)
grouped_28_order <- record.parallel(grouped_seq_28,times=48,cores=6)

# save R workspace (for running mapping on cluster computing, or prior to marker reordering)
save.image(file="readgroups_unmapped.<parent_id>.Rdata")


########################################################
#IMPORTANT NOTE:
#You can run the following code locally or using cluster computing, depending on your resources
#For cluster computing, the following code would assume 1 job per chromosome with jobs requesting 36 parallel cores
########################################################

library(BatchMap)

### read in pre-ordered, chromosome-specific marker sets
load("readgroups_unmapped.<parent_id>.Rdata")

### outcross mapping with ripple
#If running locally, you will need a seperate map_outcross object for each linkage group (e.g. map_outcross_01, map_outcross_02...)
map_outcross <- map.overlapping.batches(input.seq=grouped_XX_order,size=25,overlap=18,fun.order=ripple.ord,phase.cores=3,ripple.cores=12,method="one",min.tries=1,ws=6)

### print outcross map
map_outcross$Map


########################################################
# Excel: Copy-and-Pasting Step
# Time: Medium or Slow
########################################################
# copy and paste the output of each map_outcross object above (e.g. map_outcross_01$Map) into Excel
# You need the following columns in order: Marker, Status, Group, cM, HAP1_allele, HAP2_allele
# Fill the "Status" column with "mapped"
# Group is the linkage group ID (eg: LG1_1)
# cM is the genetic distance
# haplotype columns are the phasing output from the map_outcross object
# name this file "input_onemap.txt"

########################################################
# Perl: Add Co-segregating Markers
# Time: Fast
########################################################
# run perl script "script_06.print_map_with_cosegs.pl"
# iterates through "cosegregating_markers.txt" generated in the first step to add them back to the "input_onemap.txt" file generated above 
# prints out final genetic map as "output_onemap.txt"


